package digital.laboratory.platform.reagent.controller;

import cn.hutool.core.io.file.FileNameUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.entity.FixedValueReport;
import digital.laboratory.platform.reagent.entity.InstructionBook;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.entity.ReferenceMaterial;
import digital.laboratory.platform.reagent.service.FixedValueReportService;
import digital.laboratory.platform.reagent.service.ReferenceMaterialService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

/**
 * (定值报告)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (定值报告) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/fixed_value_report" )
@Api(value = "fixed_value_report", tags = "(定值报告)管理")
public class FixedValueReportController {

    private final  FixedValueReportService fixedValueReportService;

	private final ReferenceMaterialService referenceMaterialService;

	private final OssFile ossFile;

	/**
	 * 新增定值报告
	 *
	 * @param id 定值报告
	 * @return R
	 */
	@ApiOperation(value = "上传定值报告", notes = "上传定值报告")
	@SysLog("上传定值报告")
	@PostMapping
//    @PreAuthorize("@pms.hasPermission('fixed_value_report_add')" )
	public R<FixedValueReport> upload(String id, String fixedResult,@RequestPart(value = "file", required = false) MultipartFile file, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {

		Principal principal = theHttpServletRequest.getUserPrincipal();

		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		ReferenceMaterial byId = referenceMaterialService.getById(id);

		String path = "reagen_managment" + "/" + "fixedValueReport" + "/" + byId.getReagentConsumableId();
		String fileName = FileNameUtil.getName(file.getOriginalFilename());
		boolean b = ossFile.fileUpload(file, path);
		Map<String, String> ResultData = new HashMap<>();
		ResultData.put("fileName", fileName);
		ResultData.put("path", path);
		FixedValueReport fixedValueReport = new FixedValueReport();

		if (b) {
			fixedValueReport.setFixedValueReportId(IdWorker.get32UUID().toUpperCase());
			fixedValueReport.setFixedValueReport(fileName);
			fixedValueReport.setFixedResult(fixedResult);
			fixedValueReport.setReferenceSubstanceId(byId.getId());

			String fileNames = path + "/" + fileName;

			ossFile.fileGet(fileNames, httpServletResponse.getOutputStream());

			fixedValueReportService.save(fixedValueReport);

			return R.ok(fixedValueReport, "上传成功");
		} else return R.failed("上传失败");
	}

	/**
	 * 新增定值报告
	 *
	 * @param id 定值报告
	 * @return R
	 */
	@ApiOperation(value = "预览定值报告", notes = "预览定值报告")
	@SysLog("预览定值报告")
	@GetMapping("/download")
//    @PreAuthorize("@pms.hasPermission('reagent_instruction_book_add')" )
	public R<FixedValueReport> download(String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		ReferenceMaterial byId = referenceMaterialService.getById(id);

		LambdaQueryWrapper<FixedValueReport> fixedValueReportLambdaQueryWrapper = new LambdaQueryWrapper<>();

		fixedValueReportLambdaQueryWrapper.eq(FixedValueReport::getReferenceSubstanceId,byId.getId());

		FixedValueReport one = fixedValueReportService.getOne(fixedValueReportLambdaQueryWrapper);

		if (one != null) {

			String fileName = one.getFixedValueReport();

			String path = "reagen_managment" + "/" + "instructionBook" + "/" + byId.getReagentConsumableId();

			String fileNames = path + "/" + fileName;

			ossFile.fileGet(fileNames, httpServletResponse.getOutputStream());

			return R.ok(one);
		} else
			return R.ok(null);
	}
}
