package digital.laboratory.platform.reagent.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (试剂耗材类)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材类) 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/reagent_consumables")
@Api(value = "reagent_consumables", tags = "(试剂耗材类)管理")
public class ReagentConsumablesController {

    private final ReagentConsumablesService reagentConsumablesService;


    /**
     * 分页查询
     *
     * @param page               分页对象
     * @param name (试剂耗材类)
     * @return
     */
    @ApiOperation(value = "分页查询试剂耗材类(包括标准物质)", notes = "分页查询试剂耗材类(包括标准物质)")
    @GetMapping("/page")
//    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumables_get')" )
    public R<IPage<ReagentConsumables>> getReagentConsumablesPage(Page<ReagentConsumables> page, String name, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        IPage<ReagentConsumables> reagentConsumablesSList = reagentConsumablesService.page(page, Wrappers.<ReagentConsumables>query()
                .eq("category", "试剂").or()
                .eq("category", "耗材").or()
                .eq("category", "标准物质")
                .eq(StrUtil.isNotBlank(name),"reagent_consumable_name",name)
                .orderByDesc("create_time")
        );
        return R.ok(reagentConsumablesSList);
//		return R.ok(reagentConsumablesService.page(page, Wrappers.query(reagentConsumables)));
    }
}
