package digital.laboratory.platform.reagent.controller;

import cn.hutool.core.date.DateTime;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.annotation.JsonFormat;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.StandardSolutionCurveDTO;
import digital.laboratory.platform.reagent.entity.StandardSolutionCurve;
import digital.laboratory.platform.reagent.service.StandardSolutionCurveService;
import digital.laboratory.platform.reagent.vo.StandardSolutionCurveVO;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (标准工作曲线溶液配置记录表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准工作曲线溶液配置记录表) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/standard_solution_curve" )
@Api(value = "standard_solution_curve", tags = "(标准工作曲线溶液配置记录表)管理")
public class StandardSolutionCurveController {

    private final  StandardSolutionCurveService standardSolutionCurveService;

	/**
     * 分页查询
     * @param page 分页对象
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
//    @PreAuthorize("@pms.hasPermission('reagent_standard_solution_curve_get')" )
    public R<IPage<StandardSolutionCurveVO>> getStandardSolutionCurvePage(Page<StandardSolutionCurve> page, String solutionName, @JsonFormat(pattern = "yyyy-MM-dd")
	@DateTimeFormat(pattern = "yyyy-MM-dd")DateTime startTime,@DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") DateTime endTime, HttpServletRequest theHttpServletRequest) {

		QueryWrapper<StandardSolutionCurve> standardSolutionCurveQueryWrapper = new QueryWrapper<>();


		Principal principal = theHttpServletRequest.getUserPrincipal();

		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (solutionName!=null){

			standardSolutionCurveQueryWrapper.like("solution_name",solutionName);
		}

		if (startTime!=null&&endTime!=null){

			standardSolutionCurveQueryWrapper.ge("create_time",startTime).le("create_time",endTime);
		}

		IPage<StandardSolutionCurveVO> standardSolutionCurveVOPage = standardSolutionCurveService.getStandardSolutionCurveVOPage(page, standardSolutionCurveQueryWrapper
				.orderByDesc("create_time")
		);
		return R.ok(standardSolutionCurveVOPage);
    }


    /**
     * 新增(标准工作曲线溶液配置记录表)
     * @param standardSolutionCurveDTO (标准工作曲线溶液配置记录表)
     * @return R
     */
    @ApiOperation(value = "新增(标准工作曲线溶液配置记录表)", notes = "新增(标准工作曲线溶液配置记录表)")
    @SysLog("新增(标准工作曲线溶液配置记录表)" )
    @PostMapping
//    @PreAuthorize("@pms.hasPermission('reagent_standard_solution_curve_add')" )
    public R<StandardSolutionCurve> postAddObject(@RequestBody StandardSolutionCurveDTO standardSolutionCurveDTO, HttpServletRequest theHttpServletRequest) {

		Principal principal = theHttpServletRequest.getUserPrincipal();

		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		StandardSolutionCurve standardSolutionCurve = standardSolutionCurveService.addById(standardSolutionCurveDTO, dlpUser);

		if (standardSolutionCurve!=null){
			return R.ok(standardSolutionCurve);
		}else return R.failed("配置失败");
	}


}
