package digital.laboratory.platform.reagent.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import digital.laboratory.platform.common.mybatis.base.BaseEntity;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;
import java.time.LocalDateTime;

import lombok.Data;
import lombok.EqualsAndHashCode;


/**
 * (验收记录表)
 *
 * @author Zhang Xiaolong created at  2023-03-10 14:25:06
 * @describe (验收记录表) 实体类
 */
@Data
@TableName(value = "acceptance_record_form", autoResultMap = true)
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "(验收记录表)")
public class AcceptanceRecordForm extends BaseEntity {


    /**
     * (供应商ID)
     */
    @ApiModelProperty(value = "(供应商ID)")
    private String supplierId;

    /**
     * 部门负责人审核时间
     */
    @ApiModelProperty(value = "三级审核时间")
    private LocalDateTime auditTimeOfThreeLevel;

    /**
     * (验收日期)
     */
    @ApiModelProperty(value = "(验收日期)")
    private LocalDateTime dateOfAcceptance;

    /**
     * (部门负责人审核意见)
     */
    @ApiModelProperty(value = "(三及审核意见)")
    private String auditOpinionOfThreeLevel;

    @ApiModelProperty(value = "(提交时间)")
    private LocalDateTime commitTime;

    /**
     * (部门负责人ID)
     */
    @ApiModelProperty(value = "(三级审核人ID)")
    private String threeLevelAuditorId;

    /**
     * (后续处理情况)
     */
    @ApiModelProperty(value = "(后续处理情况)")
    private String followUpTreatment;


    /**
     * (二级审核意见)
     */
    @ApiModelProperty(value = "(二级审核意见)")
    private String auditOpinionOfSecondary;

    /**
     * (二级审核人ID)
     */
    @ApiModelProperty(value = "(二级审核人ID)")
    private String secondaryAuditorId;

    /**
     * (二级审核时间)
     */
    @ApiModelProperty(value = "(二级审核时间)")
    private LocalDateTime auditTimeOfSecondary;

    /**
     * (试剂耗材ID)
     */
    @ApiModelProperty(value = "(试剂耗材ID)")
    private String reagentConsumableId;

    /**
     * (一级审核意见)
     */
    @ApiModelProperty(value = "(一级审核意见)")
    private String auditOpinionOfPrimary;

    /**
     * (一级审核时间)
     */
    @ApiModelProperty(value = "(一级审核时间)")
    private LocalDateTime auditTimeOfPrimary;

    /**
     * (一级审核人ID)
     */
    @ApiModelProperty(value = "(一级审核人ID)")
    private String primaryAuditorId;


    /**
     * acceptanceRecordFormId
     */
    @TableId(value = "id", type = IdType.ASSIGN_UUID)
    @ApiModelProperty(value = "id")
    private String id;

    /**
     * 一级审核结果
     */
    @ApiModelProperty(value = "一级审核结果")
    private boolean auditResultOfPrimary;

    /**
     * 部门负责人审核结果
     */
    @ApiModelProperty(value = "三级审核结果")
    private boolean auditResultOfThreeLevel;

    /**
     * (二级审核结果)
     */
    @ApiModelProperty(value = "(二级审核结果)")
    private boolean auditResultOfSecondary;
    /**
     * (品牌型号一致)
     */
    @ApiModelProperty(value = "(品牌型号一致)")
    private String theSameBrandAndModel;

    /**
     * (品牌型号一致备注)
     */
    @ApiModelProperty(value = "(品牌型号一致备注)")
    private String bam_remarks;

    /**
     * (数量一致)
     */
    @ApiModelProperty(value = "(数量一致)")
    private String consistentQuantity;

    /**
     * (数量一致备注)
     */
    @ApiModelProperty(value = "(数量一致备注)")
    private String cq_remarks;
    /**
     * (包装完好)
     */
    @ApiModelProperty(value = "(包装完好)")
    private String packingInGoodCondition;
    /**
     * (包装完好备注)
     */
    @ApiModelProperty(value = "(包装完好备注)")
    private String pgc_remarks;
    /**
     * (有效期)
     */
    @ApiModelProperty(value = "(有效期)")
    private String validityPeriod;

    /**
     * (有效期备注)
     */
    @ApiModelProperty(value = "(有效期备注)")
    private String vp_remarks;

    /**
     * (供货周期)
     */
    @ApiModelProperty(value = "(供货周期)")
    private String deliveryCycle;

    /**
     * (供货周期备注)
     */
    @ApiModelProperty(value = "(供货周期备注)")
    private String  dc_remarks;

    /**
     * (验收结论)
     */
    @ApiModelProperty(value = "(验收结论)")
    private String  acceptanceConclusion;

    /**
     * (不合格项目)
     */
    @ApiModelProperty(value = "(不合格项目)")
    private String  nonconformingItem;
 /**
     * (状态)
     */
    @ApiModelProperty(value = "状态(0：未提交，1：已提交，2：一级审核通过，3：二级审核通过，4：三级审核通过)")
    private Integer  status;
/**
     * (签收物品明细Id)
     */
    @ApiModelProperty(value = "(签收物品明细Id)")
    private String   warehousingContentId;


    @ApiModelProperty(value = "(验收人名称)")
    private String  userName;


}
