package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.reagent.entity.Blacklist;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.mapper.BlacklistMapper;
import digital.laboratory.platform.reagent.service.BlacklistService;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import digital.laboratory.platform.reagent.vo.BlackListVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * (试剂耗材黑名单)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材黑名单) 服务实现类
 */
@Service
public class BlacklistServiceImpl extends ServiceImpl<BlacklistMapper, Blacklist> implements BlacklistService {

    @Autowired
    private BlacklistServiceImpl blacklistService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Override
    public Blacklist addListById(String reagentConsumableId, String supplierId) {

        Blacklist blacklist = new Blacklist();

        blacklist.setBlacklistId(IdWorker.getTimeId().toUpperCase());
        blacklist.setSupplierId(supplierId);
        blacklist.setResultsOfComplianceCheck(false);
        blacklist.setReagentConsumableId(reagentConsumableId);

        if (blacklistService.save(blacklist)) {

            return blacklist;
        } else return null;
    }

    @Override
    public Blacklist addListById2(String reagentConsumableId, String supplierId) {

        Blacklist blacklist = new Blacklist();

        blacklist.setBlacklistId(IdWorker.getTimeId().toUpperCase());
        blacklist.setSupplierId(supplierId);
        blacklist.setReagentConsumableId(reagentConsumableId);

        if (blacklistService.save(blacklist)) {

            return blacklist;
        } else return null;
    }

    @Override
    public List<BlackListVO> getBlacklist(String supplierInformationId) {

        LambdaQueryWrapper<Blacklist> blacklistLambdaQueryWrapper = new LambdaQueryWrapper<>();

        blacklistLambdaQueryWrapper.eq(Blacklist::getSupplierId, supplierInformationId);

        List<Blacklist> list = blacklistService.list(blacklistLambdaQueryWrapper);

        List<BlackListVO> blackListVOS = new ArrayList<>();

        for (Blacklist blacklist : list) {

            BlackListVO blackListVO = new BlackListVO();

            ReagentConsumables byId = reagentConsumablesService.getById(blacklist.getReagentConsumableId());

            blackListVO.setReagentConsumableName(byId.getReagentConsumableName());
            blackListVO.setCategory(byId.getCategory());
            blackListVO.setBrand(byId.getBrand());
            blackListVO.setSpecificationAndModel(byId.getSpecificationAndModel());

            blackListVOS.add(blackListVO);
        }

        return blackListVOS;
    }

}
