package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.PeriodVerificationImplementationDTO;
import digital.laboratory.platform.reagent.entity.PeriodVerificationImplementation;
import digital.laboratory.platform.reagent.entity.PeriodVerificationPlan;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.entity.ReferenceMaterial;
import digital.laboratory.platform.reagent.mapper.PeriodVerificationImplementationMapper;
import digital.laboratory.platform.reagent.service.PeriodVerificationImplementationService;
import digital.laboratory.platform.reagent.service.PeriodVerificationPlanService;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import digital.laboratory.platform.reagent.service.ReferenceMaterialService;
import digital.laboratory.platform.reagent.vo.PeriodVerificationImplementationVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;

/**
 * (标准物质期间核查实施情况及结果记录表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质期间核查实施情况及结果记录表) 服务实现类
 */
@Service
public class PeriodVerificationImplementationServiceImpl extends ServiceImpl<PeriodVerificationImplementationMapper, PeriodVerificationImplementation> implements PeriodVerificationImplementationService {
    @Autowired
    private PeriodVerificationPlanService periodVerificationPlanService;

    @Autowired
    private PeriodVerificationImplementationService periodVerificationImplementationService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;
    @Autowired

    private ReferenceMaterialService referenceMaterialService;

    @Override//分页查询
    public IPage<PeriodVerificationImplementationVO> getPeriodVerificationImplementationVOPage(IPage<PeriodVerificationImplementation> page, QueryWrapper<PeriodVerificationImplementation> qw) {

        IPage<PeriodVerificationImplementationVO> periodVerificationImplementationVOPage = baseMapper.getPeriodVerificationImplementationVOPage(page, qw);

        return periodVerificationImplementationVOPage;
    }

    @Override//通过ID查询
    public PeriodVerificationImplementationVO getPeriodVerificationImplementationVO(String periodVerificationImplementationId) {

        PeriodVerificationImplementationVO periodVerificationImplementationVO = baseMapper.getPeriodVerificationImplementationVO(periodVerificationImplementationId);

        return periodVerificationImplementationVO;
    }


    @Override//新增期间核查结果录入表
    public PeriodVerificationImplementation addById(PeriodVerificationPlan periodVerificationPlan) {

        PeriodVerificationImplementation periodVerificationImplementation = new PeriodVerificationImplementation();

        periodVerificationImplementation.setId(IdWorker.get32UUID().toUpperCase());
        //赋值信息
        periodVerificationImplementation.setDeviationAndUncertainty(periodVerificationPlan.getDeviationAndUncertainty());
        periodVerificationImplementation.setReferenceMaterialId(periodVerificationPlan.getReferenceMaterialId());
        periodVerificationImplementation.setReferenceMaterialNumber(periodVerificationPlan.getReferenceMaterialNumber());
        periodVerificationImplementation.setPeriodVerificationPlanId(periodVerificationPlan.getId());

        LambdaQueryWrapper<ReferenceMaterial> referenceMaterialLambdaQueryWrapper = new LambdaQueryWrapper<>();
        referenceMaterialLambdaQueryWrapper.eq(ReferenceMaterial::getId, periodVerificationPlan.getReferenceMaterialId());
        ReferenceMaterial referenceMaterial = referenceMaterialService.getOne(referenceMaterialLambdaQueryWrapper);

        ReagentConsumables reagentConsumables = reagentConsumablesService.getById(referenceMaterial.getReagentConsumableId());
        //赋值信息
        periodVerificationImplementation.setReferenceMaterialName(reagentConsumables.getReagentConsumableName());
        periodVerificationImplementation.setStandardValueOrPurity(reagentConsumables.getStandardValueOrPurity());
        periodVerificationImplementation.setVerificationBasis(periodVerificationPlan.getVerificationBasis());
        periodVerificationImplementation.setCommitStatus(0);

        if (periodVerificationImplementationService.save(periodVerificationImplementation)) {
            return periodVerificationImplementation;
        } else throw new RuntimeException(String.format("保存失败"));

    }

    @Override//录入期间核查结果记录表
    public PeriodVerificationImplementation editById(PeriodVerificationImplementationDTO periodVerificationImplementationDTO, DLPUser dlpUser) {

        PeriodVerificationImplementation byId = periodVerificationImplementationService.getById(periodVerificationImplementationDTO.getPeriodVerificationImplementationId());

        BeanUtils.copyProperties(periodVerificationImplementationDTO, byId);

        byId.setInspectorId(dlpUser.getId());

        if (periodVerificationImplementationDTO.getCauseOfDissatisfaction() != null) {

            byId.setOpinionOfInspector(false);

        } else byId.setOpinionOfInspector(true);

        ReferenceMaterial byId1 = referenceMaterialService.getById(byId.getReferenceMaterialId());

        ReagentConsumables byId2 = reagentConsumablesService.getById(byId1.getReagentConsumableId());

        byId2.setDeviationOrUncertainty(periodVerificationImplementationDTO.getDeviationAndUncertainty());

        reagentConsumablesService.updateById(byId2);

        String result = periodVerificationImplementationDTO.getResult();

        byId.setCheckingTime(LocalDate.now());

        PeriodVerificationPlan periodVerificationPlan = periodVerificationPlanService.getById(periodVerificationImplementationDTO.getPeriodVerificationPlanId());

        periodVerificationPlan.setVerificationResult(result);

        periodVerificationPlan.setImplementationDate(byId.getCheckingTime());

        periodVerificationPlan.setInspectorId(dlpUser.getId());

        if (periodVerificationImplementationService.updateById(byId) && periodVerificationPlanService
                .updateById(periodVerificationPlan)) {

            return byId;
        } else throw new RuntimeException(String.format("保存失败"));

    }

    @Override//提交期间核查结果记录表
    public PeriodVerificationImplementation commitById(PeriodVerificationImplementationDTO periodVerificationImplementationDTO, DLPUser dlpUser) {

        PeriodVerificationImplementation periodVerificationImplementation = periodVerificationImplementationService.editById(periodVerificationImplementationDTO, dlpUser);

        periodVerificationImplementation.setCommitStatus(1);

        periodVerificationImplementation.setCommitTime(LocalDateTime.now());

        if (periodVerificationImplementationService.updateById(periodVerificationImplementation)) {

            return periodVerificationImplementation;

        } else return null;
    }

    @Override//审核期间核查结果记录表
    public PeriodVerificationImplementation auditById(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        PeriodVerificationImplementation byId = periodVerificationImplementationService.getById(auditAndApproveDTO.getUuId());

        byId.setAuditResultOfTechnical(auditAndApproveDTO.getAuditResult());
        byId.setAuditOpinionOfTechnical(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditTimeOfTechnical(LocalDate.now());
        byId.setTechnicalDirectorId(dlpUser.getId());

        if (auditAndApproveDTO.getAuditResult() == true) {

            byId.setCommitStatus(2);
        } else byId.setCommitStatus(-2);

        if (periodVerificationImplementationService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("审核失败"));
    }
}
