package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.reagent.entity.EvaluationForm;
import digital.laboratory.platform.reagent.entity.ProvideServicesOrSupplies;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.mapper.ProvideServicesOrSuppliesMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.ProvideServicesOrSuppliesVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 提供服务或供应品服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 提供服务或供应品 服务实现类
 */
@Service
public class ProvideServicesOrSuppliesServiceImpl extends ServiceImpl<ProvideServicesOrSuppliesMapper, ProvideServicesOrSupplies> implements ProvideServicesOrSuppliesService {

    @Autowired
    private ProvideServicesOrSuppliesService provideServicesOrSuppliesService;

    @Autowired
    private EvaluationFormService evaluationFormService;

    @Autowired
    private SupplierInformationService supplierInformationService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Override
    public ProvideServicesOrSupplies addById(String evaluationFormId,String reagentConsumableId){

        ProvideServicesOrSupplies provideServicesOrSupplies = new ProvideServicesOrSupplies();

        provideServicesOrSupplies.setId(IdWorker.get32UUID().toUpperCase());

        EvaluationForm one = evaluationFormService.getById(evaluationFormId);

        provideServicesOrSupplies.setEvaluationFormId(one.getId());

        provideServicesOrSupplies.setReagentConsumableId(reagentConsumableId);

        provideServicesOrSuppliesService.save(provideServicesOrSupplies);

        return provideServicesOrSupplies;

    }
    @Override
    public List<ProvideServicesOrSuppliesVO> getVOList(String evaluationFormId){
        //通过供应商ID找出对应的评价表
        EvaluationForm one = evaluationFormService.getById(evaluationFormId);

        LambdaQueryWrapper<ProvideServicesOrSupplies> provideServicesOrSuppliesLambdaQueryWrapper = new LambdaQueryWrapper<>();

        provideServicesOrSuppliesLambdaQueryWrapper.eq(ProvideServicesOrSupplies::getEvaluationFormId,one.getId());
        //通过评价表ID，找出该供应商提供的所有供应品
        List<ProvideServicesOrSupplies> list = provideServicesOrSuppliesService.list(provideServicesOrSuppliesLambdaQueryWrapper);

        List<ProvideServicesOrSuppliesVO> provideServicesOrSuppliesVOS = new ArrayList<>();

        LambdaQueryWrapper<ReagentConsumables> reagentConsumablesLambdaQueryWrapper = new LambdaQueryWrapper<>();

        for (ProvideServicesOrSupplies provideServicesOrSupplies : list) {

            ProvideServicesOrSuppliesVO provideServicesOrSuppliesVO = new ProvideServicesOrSuppliesVO();

            BeanUtils.copyProperties(provideServicesOrSupplies,provideServicesOrSuppliesVO);

            ReagentConsumables reagentConsumables = reagentConsumablesService.getById(provideServicesOrSuppliesVO.getReagentConsumableId());
            //赋值服务品VO信息
            provideServicesOrSuppliesVO.setCategory(reagentConsumables.getCategory());
            provideServicesOrSuppliesVO.setBrand(reagentConsumables.getBrand());
            provideServicesOrSuppliesVO.setSpecificationAndModel(reagentConsumables.getSpecificationAndModel());
            provideServicesOrSuppliesVO.setReagentConsumableName(reagentConsumables.getReagentConsumableName());

            provideServicesOrSuppliesVOS.add(provideServicesOrSuppliesVO);
        }
        return provideServicesOrSuppliesVOS;
    }
}
