package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.DateTime;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.PurchaseListDTO;
import digital.laboratory.platform.reagent.dto.PurchasingPlanDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.PurchasingPlanMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.ProcurementContentVO;
import digital.laboratory.platform.reagent.vo.PurchasingPlanVO;
import digital.laboratory.platform.reagent.vo.ReplacementVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * (采购计划)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购计划) 服务实现类
 */
@Service
public class PurchasingPlanServiceImpl extends ServiceImpl<PurchasingPlanMapper, PurchasingPlan> implements PurchasingPlanService {

    @Autowired
    private PurchasingPlanService purchasingPlanService;
    @Autowired
    private ProcurementContentService procurementContentService;

    @Autowired
    private DetailsOfCentralizedService detailsOfCentralizedService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private PurchaseListService purchaseListService;

    @Autowired
    private CentralizedRequestService requestService;

    @Autowired
    private PurchaseListDetailsService purchaseListDetailsService;

    @Autowired
    private WarehousingContentService warehousingContentService;

    @Autowired
    private SupplierInformationService supplierInformationService;

    @Override//分页
    public IPage<PurchasingPlanVO> getPurchasingPlanVOPage(IPage<PurchasingPlan> page, QueryWrapper<PurchasingPlan> qw) {
        IPage<PurchasingPlanVO> purchasingPlanVOPage = baseMapper.getPurchasingPlanVOPage(page, qw);
        return purchasingPlanVOPage;
    }

    @Override//列表
    public List<PurchasingPlanVO> getPurchasingPlanVOList(QueryWrapper<PurchasingPlan> qw) {
        return baseMapper.getPurchasingPlanVOList(qw);
    }

    @Override//通过ID查询
    public PurchasingPlanVO getPurchasingPlanVO(String purchasingPlanId) {

        PurchasingPlanVO purchasingPlanVO = baseMapper.getPurchasingPlanVO(purchasingPlanId);

        if (purchasingPlanVO == null) {

            throw new RuntimeException(String.format("信息不存在"));
        }

        List<ProcurementContentVO> procurementContentVOList = procurementContentService.getProcurementContentVOList(purchasingPlanId);

        purchasingPlanVO.setProcurementContentVOList(procurementContentVOList);

        return purchasingPlanVO;
    }

    @Transactional
    @Override//创建计划
    public PurchasingPlanVO addById(List<PurchasingPlanDTO> purchasingPlanDTOList, DLPUser user) {

        PurchasingPlan purchasingPlan = new PurchasingPlan();

        purchasingPlan.setPurchasingPlanId(IdWorker.get32UUID().toUpperCase());

        purchasingPlan.setCreateId(user.getId());

        purchasingPlan.setStatus(0);

        purchasingPlan.setCreateName(user.getName());

        List<ProcurementContent> list = new ArrayList<>();//需要整合的集合

        boolean flag = true;

        for (PurchasingPlanDTO purchasingPlanDTO : purchasingPlanDTOList) {
            //将列入计划的集中采购申请状态变为3
            String detailsOfCentralizedId = purchasingPlanDTO.getDetailsOfCentralizedId();

            DetailsOfCentralized byId2 = detailsOfCentralizedService.getById(detailsOfCentralizedId);

            CentralizedRequest byId3 = requestService.getById(byId2.getCentralizedRequestId());

            byId3.setStatus(3);

            requestService.updateById(byId3);

            flag = true;
//
            //合并相同的采购内容
            if (list.size() != 0) {

                for (int i = 0; i < list.size(); i++) {

                    if (list.get(i).getReagentConsumableId().equals(purchasingPlanDTO.getReagentConsumableId())) {

                        ReagentConsumables byId1 = reagentConsumablesService.getById(list.get(i).getReagentConsumableId());

                        list.get(i).setUnitPrice(byId1.getUnitPrice());
                        //Set采购数量
                        list.get(i).setQuantityPurchased(list.get(i).getQuantityPurchased() + purchasingPlanDTO.getNumberOfApplications());

                        list.get(i).setSubtotal(list.get(i).getQuantityPurchased() * list.get(i).getUnitPrice());

                        DetailsOfCentralized byId = detailsOfCentralizedService.getById(purchasingPlanDTO.getDetailsOfCentralizedId());

                        byId.setQuantityPurchased(byId.getNumberOfApplications());

                        byId.setProcurementContentId(list.get(i).getProcurementContentId());

                        detailsOfCentralizedService.updateById(byId);

                        flag = false;

                    }
                }
                if (flag) {
                    ProcurementContent procurementContent = new ProcurementContent();

                    BeanUtils.copyProperties(purchasingPlanDTO, procurementContent);

                    procurementContent.setProcurementContentId(IdWorker.get32UUID().toUpperCase());

                    procurementContent.setPurchasingPlanId(purchasingPlan.getPurchasingPlanId());

                    procurementContent.setQuantityPurchased(purchasingPlanDTO.getNumberOfApplications());

                    DetailsOfCentralized byId = detailsOfCentralizedService.getById(purchasingPlanDTO.getDetailsOfCentralizedId());

                    byId.setQuantityPurchased(byId.getNumberOfApplications());

                    ReagentConsumables reagentConsumables = reagentConsumablesService.getById(byId.getReagentConsumableId());

                    procurementContent.setUnitPrice(reagentConsumables.getUnitPrice());

                    procurementContent.setSubtotal(procurementContent.getUnitPrice() * procurementContent.getQuantityPurchased());

                    byId.setProcurementContentId(procurementContent.getProcurementContentId());

                    detailsOfCentralizedService.updateById(byId);

                    list.add(procurementContent);

                }

            } else {
                ProcurementContent procurementContent = new ProcurementContent();

                BeanUtils.copyProperties(purchasingPlanDTO, procurementContent);

                DetailsOfCentralized byId = detailsOfCentralizedService.getById(purchasingPlanDTO.getDetailsOfCentralizedId());

                byId.setQuantityPurchased(byId.getNumberOfApplications());

                procurementContent.setProcurementContentId(IdWorker.get32UUID().toUpperCase());

                procurementContent.setPurchasingPlanId(purchasingPlan.getPurchasingPlanId());

                procurementContent.setQuantityPurchased(procurementContent.getNumberOfApplications());

                ReagentConsumables reagentConsumables = reagentConsumablesService.getById(byId.getReagentConsumableId());

                procurementContent.setUnitPrice(reagentConsumables.getUnitPrice());

                procurementContent.setSubtotal(procurementContent.getQuantityPurchased() * procurementContent.getUnitPrice());

                byId.setProcurementContentId(procurementContent.getProcurementContentId());

                detailsOfCentralizedService.updateById(byId);

                list.add(procurementContent);
            }
        }
        double x = 0;
        for (
                ProcurementContent procurementContent : list) {

            x = x + procurementContent.getNumberOfApplications() * procurementContent.getUnitPrice();

        }

        purchasingPlan.setAppropriationBudget(x);
        if (purchasingPlanService.save(purchasingPlan) & procurementContentService.saveBatch(list)) {

            PurchasingPlanVO purchasingPlanVO = purchasingPlanService.getPurchasingPlanVO(purchasingPlan.getPurchasingPlanId());
            return purchasingPlanVO;
        } else throw new RuntimeException(String.format("保存失败"));
    }

    @Override
    public ProcurementContent addContent(PurchasingPlanDTO purchasingPlanDTO) {

        ProcurementContent procurementContent = new ProcurementContent();

        BeanUtils.copyProperties(purchasingPlanDTO, procurementContent);

        if (procurementContentService.save(procurementContent)) {
            return procurementContent;
        } else {
            return null;
        }
    }

    @Override
    @Transactional
    public ProcurementContent editById(PurchasingPlanDTO purchasingPlanDTO) {

        ProcurementContent byId = procurementContentService.getById(purchasingPlanDTO.getProcurementContentId());

        BeanUtils.copyProperties(purchasingPlanDTO, byId);

        byId.setSubtotal(byId.getUnitPrice() * byId.getQuantityPurchased());

        LambdaQueryWrapper<DetailsOfCentralized> detailsOfCentralizedLambdaQueryWrapper = new LambdaQueryWrapper<>();

        detailsOfCentralizedLambdaQueryWrapper.eq(DetailsOfCentralized::getId, byId.getDetailsOfCentralizedId());

        DetailsOfCentralized detailsOfCentralized = detailsOfCentralizedService.getOne(detailsOfCentralizedLambdaQueryWrapper);

        detailsOfCentralized.setQuantityPurchased(byId.getQuantityPurchased());

        procurementContentService.updateById(byId);

        detailsOfCentralizedService.updateById(detailsOfCentralized);

        calculatedAmount(byId.getPurchasingPlanId());

        return byId;

    }


    @Override//修改整合的采购计划明细中的申请明细

    public ProcurementContent editDetails(PurchasingPlanDTO purchasingPlanDTO) {

        ProcurementContent byId = procurementContentService.getById(purchasingPlanDTO.getProcurementContentId());

        PurchasingPlan purchasingPlan = purchasingPlanService.getById(byId.getPurchasingPlanId());

        DetailsOfCentralized detailsOfCentralized = detailsOfCentralizedService.getById(purchasingPlanDTO.getDetailsOfCentralizedId());

        if (detailsOfCentralized.getQuantityPurchased() == null) {

            byId.setQuantityPurchased(byId.getQuantityPurchased() - detailsOfCentralized.getNumberOfApplications() + purchasingPlanDTO.getQuantityPurchased());

        } else {

            byId.setQuantityPurchased(byId.getQuantityPurchased() - detailsOfCentralized.getQuantityPurchased() + purchasingPlanDTO.getQuantityPurchased());
        }

        byId.setSubtotal(byId.getUnitPrice() * byId.getQuantityPurchased());

        detailsOfCentralized.setQuantityPurchased(purchasingPlanDTO.getQuantityPurchased());

        detailsOfCentralized.setDetailsRemark(purchasingPlanDTO.getPlanRemark());

        calculatedAmount(byId.getPurchasingPlanId());

        if (procurementContentService.updateById(byId) && detailsOfCentralizedService.updateById(detailsOfCentralized)) {

            calculatedAmount(byId.getPurchasingPlanId());

            return byId;

        } else throw new RuntimeException(String.format("修改失败"));
    }


    @Override
    @Transactional
    public Boolean delContentById(String procurementContentId) {

        ProcurementContent byId = procurementContentService.getById(procurementContentId);

        LambdaQueryWrapper<DetailsOfCentralized> detailsOfCentralizedLambdaQueryWrapper = new LambdaQueryWrapper<>();

        detailsOfCentralizedLambdaQueryWrapper.eq(DetailsOfCentralized::getProcurementContentId, procurementContentId);

        List<DetailsOfCentralized> list = detailsOfCentralizedService.list(detailsOfCentralizedLambdaQueryWrapper);

        for (DetailsOfCentralized detailsOfCentralized : list) {

            detailsOfCentralized.setQuantityPurchased(0);
        }
        PurchasingPlan purchasingPlan = purchasingPlanService.getById(byId.getPurchasingPlanId());

        if (purchasingPlanService.getById(byId.getPurchasingPlanId()).getStatus() == 0
                && detailsOfCentralizedService.updateBatchById(list) && procurementContentService.removeById(procurementContentId)) {

            LambdaQueryWrapper<ProcurementContent> procurementContentLambdaQueryWrapper = new LambdaQueryWrapper<>();

            procurementContentLambdaQueryWrapper.eq(ProcurementContent::getPurchasingPlanId, purchasingPlan.getPurchasingPlanId());

            List<ProcurementContent> list1 = procurementContentService.list(procurementContentLambdaQueryWrapper);

            if (list1.size() == 0) {

                purchasingPlanService.removeById(purchasingPlan);

                return null;
            }
            calculatedAmount(byId.getPurchasingPlanId());

            return true;
        } else {
            return null;
        }
    }

    @Override//换货
    public ProcurementContent replacementById(PurchasingPlanDTO purchasingPlanDTO) {

        String replacementReagentConsumableId = purchasingPlanDTO.getReplacementReagentConsumableId();

        ProcurementContent byId = procurementContentService.getById(purchasingPlanDTO.getProcurementContentId());
        //将换货信息补充至采购计划明细
        byId.setReplacementReagentConsumableId(replacementReagentConsumableId);
        byId.setReplacementQuantity(purchasingPlanDTO.getReplacementQuantity());
        byId.setReplacementPrice(purchasingPlanDTO.getReplacementPrice());
        byId.setReplacementRemark(purchasingPlanDTO.getReplacementRemark());
        byId.setSupplierId(purchasingPlanDTO.getSupplierId());

        LambdaQueryWrapper<DetailsOfCentralized> detailsOfCentralizedLambdaQueryWrapper = new LambdaQueryWrapper<>();

        detailsOfCentralizedLambdaQueryWrapper.eq(DetailsOfCentralized::getProcurementContentId, byId.getProcurementContentId());

        List<DetailsOfCentralized> list = detailsOfCentralizedService.list(detailsOfCentralizedLambdaQueryWrapper);

        LambdaQueryWrapper<PurchaseListDetails> purchaseListDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

        purchaseListDetailsLambdaQueryWrapper.eq(PurchaseListDetails::getProcurementContentId, byId.getProcurementContentId());

        PurchaseListDetails one = purchaseListDetailsService.getOne(purchaseListDetailsLambdaQueryWrapper);
        //如果存在采购清单，则需修改清单信息
        if (one != null) {

            LambdaQueryWrapper<WarehousingContent> warehousingContentLambdaQueryWrapper = new LambdaQueryWrapper<>();

            warehousingContentLambdaQueryWrapper.eq(WarehousingContent::getPurchaseListDetailsId, one.getId());

            one.setSupplierId(purchasingPlanDTO.getSupplierId());
            one.setPurchaseQuantity(purchasingPlanDTO.getReplacementQuantity());
            one.setReagentConsumableId(purchasingPlanDTO.getReplacementReagentConsumableId());

            purchaseListDetailsService.updateById(one);

            WarehousingContent one1 = warehousingContentService.getOne(warehousingContentLambdaQueryWrapper);
            //若存在入库信息，则需修改入库记录
            if (one1 != null) {

                if (one1.getWarehousingQuantity() == 0) {

                    one1.setReagentConsumableId(purchasingPlanDTO.getReplacementReagentConsumableId());
                    one1.setTotalQuantity(purchasingPlanDTO.getReplacementQuantity());
                    one1.setSupplierId(purchasingPlanDTO.getSupplierId());
                    ReagentConsumables byId1 = reagentConsumablesService.getById(purchasingPlanDTO.getReplacementReagentConsumableId());
                    one1.setReagentConsumableName(byId1.getReagentConsumableName());
                    warehousingContentService.updateById(one1);
                } else {

                    throw new RuntimeException(String.format("该物品已被签收入库，无法进行换货处理"));
                }
            }

        }

        for (DetailsOfCentralized detailsOfCentralized : list) {

            detailsOfCentralized.setReplacementReagentConsumableId(replacementReagentConsumableId);
            detailsOfCentralized.setReplacementQuantity(purchasingPlanDTO.getReplacementQuantity());
            detailsOfCentralized.setReplacementRemark(purchasingPlanDTO.getReplacementRemark());

            detailsOfCentralizedService.updateById(detailsOfCentralized);
        }
        //将换货信息补充至集中采购申请明细


        if (procurementContentService.updateById(byId)) {

            return byId;

        } else throw new RuntimeException(String.format("换货失败"));


    }

    @Override
    public PurchasingPlan commitById(String purchasingPlanId) {


        PurchasingPlan byId = purchasingPlanService.getById(purchasingPlanId);

        if (byId.getStatus()!=0){
            throw new RuntimeException(String.format("当前状态无法提交"));
        }

        byId.setStatus(1);
        byId.setCommitTime(LocalDateTime.now());


        if (purchasingPlanService.updateById(byId)) {
            calculatedAmount(byId.getPurchasingPlanId());
            return byId;
        } else throw new RuntimeException(String.format("提交失败"));

    }


    @Override
    public PurchasingPlan auditById(AuditAndApproveDTO auditAndApproveDto, DLPUser dlpUser) {

        PurchasingPlan purchasingPlan = purchasingPlanService.getById(auditAndApproveDto.getUuId());

        if (purchasingPlan.getStatus() != 1) {
            throw new RuntimeException(String.format("当前状态不可审核"));
        }
        purchasingPlan.setAuditResultOfPrimary(auditAndApproveDto.getAuditResult());

        purchasingPlan.setAuditOpinionOfPrimary(auditAndApproveDto.getAuditOpinion());

        purchasingPlan.setAuditTimeOfPrimary(LocalDateTime.now());

        purchasingPlan.setPrimaryAuditorId(dlpUser.getId());

        if (auditAndApproveDto.getAuditResult() == true) {
            purchasingPlan.setStatus(4);
        } else {
            purchasingPlan.setStatus(-4);
        }

        if (purchasingPlanService.updateById(purchasingPlan)) {
            return purchasingPlan;
        } else {
            return null;
        }
    }

    @Override
    public PurchasingPlan approveById(AuditAndApproveDTO auditAndApproveDto, DLPUser dlpUser) {

        PurchasingPlan purchasingPlan = purchasingPlanService.getById(auditAndApproveDto.getUuId());

        if (purchasingPlan.getStatus() != 4) {
            throw new RuntimeException(String.format("当前状态不可审批"));
        }


        purchasingPlan.setApprovalResult(auditAndApproveDto.getApproveResult());

        purchasingPlan.setApprovalOpinion(auditAndApproveDto.getApproveOpinion());

        purchasingPlan.setApprovalTime(LocalDateTime.now());

        purchasingPlan.setApproverId(dlpUser.getId());
        //审批通过则创建采购清单
        if (auditAndApproveDto.getApproveResult() == true) {

            purchasingPlan.setStatus(5);

            LambdaQueryWrapper<ProcurementContent> procurementContentLambdaQueryWrapper = new LambdaQueryWrapper<>();

            procurementContentLambdaQueryWrapper.eq(ProcurementContent::getPurchasingPlanId, purchasingPlan.getPurchasingPlanId());

            List<ProcurementContent> list = procurementContentService.list(procurementContentLambdaQueryWrapper);

            PurchaseListDTO purchaseListDTO = new PurchaseListDTO();

            PurchaseList purchaseList = new PurchaseList();
            purchaseList.setId(IdWorker.get32UUID().toUpperCase());
            purchaseList.setName(DateTime.now() + "采购清单");
            purchaseList.setStatus(0);
            purchaseList.setType("采购计划");

            purchaseListService.save(purchaseList);

            for (ProcurementContent procurementContent : list) {

                PurchaseListDetails purchaseListDetails = new PurchaseListDetails();

                purchaseListDetails.setPurchaseListId(purchaseList.getId());
                purchaseListDetails.setPurchaseQuantity(procurementContent.getQuantityPurchased());
                purchaseListDetails.setReagentConsumableId(procurementContent.getReagentConsumableId());
                purchaseListDetails.setId(IdWorker.get32UUID().toUpperCase());
                purchaseListDetails.setProcurementContentId(procurementContent.getProcurementContentId());

                purchaseListDetailsService.save(purchaseListDetails);
            }
        } else {

            purchasingPlan.setStatus(-5);
        }

        if (purchasingPlanService.updateById(purchasingPlan)) {
            return purchasingPlan;
        } else {
            return null;
        }
    }

    //计算经费预算方法
    public PurchasingPlan calculatedAmount(String purchasingPlanId) {

        PurchasingPlan purchasingPlan = purchasingPlanService.getById(purchasingPlanId);

        LambdaQueryWrapper<ProcurementContent> procurementContentLambdaQueryWrapper = new LambdaQueryWrapper<>();

        procurementContentLambdaQueryWrapper.eq(ProcurementContent::getPurchasingPlanId, purchasingPlan.getPurchasingPlanId());

        List<ProcurementContent> list = procurementContentService.list(procurementContentLambdaQueryWrapper);

        double x = 0;
        for (
                ProcurementContent procurementContent : list) {

            x = x + procurementContent.getQuantityPurchased() * procurementContent.getUnitPrice();
        }
        purchasingPlan.setAppropriationBudget(x);

        if (purchasingPlanService.updateById(purchasingPlan)) {
            return purchasingPlan;
        } else throw new RuntimeException(String.format("计算失败"));
    }

    @Override
    public ReplacementVO getReplacementVO(String id){

        ProcurementContent byId = procurementContentService.getById(id);

        ReplacementVO replacementVO = new ReplacementVO();

        if (byId.getReplacementQuantity()!=null&&byId.getReplacementPrice()!=null&&byId.getReplacementRemark()!=null
        &&byId.getReplacementReagentConsumableId()!=null&&byId.getSupplierId()!=null){

        replacementVO.setReplacementQuantity(byId.getReplacementQuantity());
        replacementVO.setReplacementRemark(byId.getReplacementRemark());
        replacementVO.setReplacementPrice(byId.getReplacementPrice());
        replacementVO.setReplacementReagentConsumableId(byId.getReplacementReagentConsumableId());
        replacementVO.setSupplierId(byId.getSupplierId());

        SupplierInformation byId1 = supplierInformationService.getById(byId.getSupplierId());

        replacementVO.setSupplierName(byId1.getSupplierName());

        ReagentConsumables byId2 = reagentConsumablesService.getById(byId.getReplacementReagentConsumableId());

        replacementVO.setReagentConsumables(byId2);}

        return replacementVO;

    }
}
