package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.reagent.config.PageUtils;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.ReagentConsumableInventoryMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.BatchDetailsVO;
import digital.laboratory.platform.reagent.vo.ReagentConsumableInventoryFullVO;
import digital.laboratory.platform.reagent.vo.ReagentConsumableInventoryVO;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 试剂耗材库存服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 试剂耗材库存 服务实现类
 */
@Service
@SuppressWarnings("all")
public class ReagentConsumableInventoryServiceImpl extends ServiceImpl<ReagentConsumableInventoryMapper, ReagentConsumableInventory> implements ReagentConsumableInventoryService {

    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;
    @Autowired
    private BatchDetailsService batchDetailsService;
    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private SupplierInformationService supplierInformationService;

    @Autowired
    private FixedValueReportService fixedValueReportService;

    @Autowired
    private StandardReserveSolutionService standardReserveSolutionService;

    @Override//试剂耗材管理列表
    public Page<ReagentConsumableInventoryVO> getReagentConsumableInventoryVOList(Page page, String name) {

        QueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryQueryWrapper = new QueryWrapper<>();

        reagentConsumableInventoryQueryWrapper.eq("status", 1);

        if (StrUtil.isNotBlank(name)) {

            reagentConsumableInventoryQueryWrapper
                    .eq("category", "试剂").or()
                    .eq("category", "耗材")
                    .like("reagent_consumable_name", name);
        } else {

            reagentConsumableInventoryQueryWrapper.eq("category", "试剂").or()
                    .eq("category", "耗材");
        }

        List<ReagentConsumableInventory> list = reagentConsumableInventoryService.list(reagentConsumableInventoryQueryWrapper);

        ArrayList<ReagentConsumableInventoryVO> reagentConsumableInventoryVOS = new ArrayList<>();

        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            List<BatchDetailsVO> batchDetailsVOList = batchDetailsService.getBatchDetailsList(reagentConsumableInventory.getReagentConsumableInventoryId());

            ReagentConsumableInventoryVO reagentConsumableInventoryVO = new ReagentConsumableInventoryVO();

            BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryVO);
            //赋值试剂耗材批次明细
            reagentConsumableInventoryVO.setBatchDetailsVOS(batchDetailsVOList);

            ReagentConsumables byId = reagentConsumablesService.getById(reagentConsumableInventory.getReagentConsumableId());
            //赋值试剂耗材名称
            reagentConsumableInventoryVO.setReagentConsumablesName(byId.getReagentConsumableName());

            reagentConsumableInventoryVOS.add(reagentConsumableInventoryVO);
        }
        if (reagentConsumableInventoryVOS != null) {
            PageUtils pageUtils = new PageUtils();

            Page pages = pageUtils.getPages((int) page.getCurrent(), (int) page.getSize(), reagentConsumableInventoryVOS);

            return pages;
        } else return null;

    }

    @Override//标准物质管理列表
    public Page<ReagentConsumableInventoryVO> getReagentConsumableInventoryRMVOList(Page page, String reagentConsumableName) {

        List<ReagentConsumableInventory> list = reagentConsumableInventoryService.list( Wrappers.<ReagentConsumableInventory>query()
                .eq("category", "标准物质").or()
                .eq("category", "标准储备溶液")
                .like(StrUtil.isNotBlank(reagentConsumableName),"reagent_consumable_name",reagentConsumableName));


        ArrayList<ReagentConsumableInventoryVO> reagentConsumableInventoryVOS = new ArrayList<>();

        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            List<BatchDetailsVO> batchDetailsVOList = batchDetailsService.getBatchDetailsList(reagentConsumableInventory.getReagentConsumableInventoryId());

            ReagentConsumableInventoryVO reagentConsumableInventoryVO = new ReagentConsumableInventoryVO();

            BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryVO);
            //赋值试剂耗材批次明细
            reagentConsumableInventoryVO.setBatchDetailsVOS(batchDetailsVOList);

            ReagentConsumables byId = reagentConsumablesService.getById(reagentConsumableInventory.getReagentConsumableId());
            //赋值试剂耗材名称
            reagentConsumableInventoryVO.setReagentConsumablesName(byId.getReagentConsumableName());

            reagentConsumableInventoryVOS.add(reagentConsumableInventoryVO);
        }
        if (reagentConsumableInventoryVOS != null) {
            PageUtils pageUtils = new PageUtils();

            Page pages = pageUtils.getPages((int) page.getCurrent(), (int) page.getSize(), reagentConsumableInventoryVOS);

            return pages;
        } else return null;
    }


    @Override//入库增加库存
    public ReagentConsumableInventory addById(String reagentConsumableId, Integer quantity) {

        LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, reagentConsumableId);

        ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);

        one.setTotalQuantity(one.getTotalQuantity() + quantity);


        if (reagentConsumableInventoryService.updateById(one)) {

            return one;
        } else throw new RuntimeException(String.format("库存添加失败"));
    }

    @Override//领用减少库存
    public ReagentConsumableInventory reduceById(String reagentConsumableId, Integer quantity) {

        LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, reagentConsumableId);

        ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);

        one.setTotalQuantity(one.getTotalQuantity() - quantity);
        if (one.getTotalQuantity() == 0) {

            one.setStatus(-1);
        }
        if (reagentConsumableInventoryService.updateById(one)) {

            return one;
        } else throw new RuntimeException(String.format("库存减少失败"));
    }


    @Override//标准物质/溶液停用
    public ReferenceMaterial deactivateById(String referenceMaterialId) {

        ReferenceMaterial byId = referenceMaterialService.getById(referenceMaterialId);

        byId.setStatus(-1);

        return byId;
    }

    @Override//分页查询标准物质
    public Page<ReagentConsumableInventoryFullVO> getAllList(Integer current, Integer size, QueryWrapper<ReagentConsumableInventory> qw, Integer status) {

        List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOList = new ArrayList<>();

        List<ReagentConsumableInventory> list = reagentConsumableInventoryService.list(qw);

        ArrayList<ReferenceMaterial> referenceMaterialList = new ArrayList<>();
        //通过仓库类信息，查找对应的该类所有标准物质
        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            if (reagentConsumableInventory.getTotalQuantity() > 0) {

                LambdaQueryWrapper<ReferenceMaterial> referenceMaterialLambdaQueryWrapper = new LambdaQueryWrapper<>();

                if (status != null) {

                    referenceMaterialLambdaQueryWrapper.eq(ReferenceMaterial::getStatus, status).
                            eq(ReferenceMaterial::getReagentConsumableInventoryId, reagentConsumableInventory.getReagentConsumableInventoryId())
                            .ne(ReferenceMaterial::getStatus, -4);
                } else {
                    referenceMaterialLambdaQueryWrapper.eq(ReferenceMaterial::getReagentConsumableInventoryId, reagentConsumableInventory.getReagentConsumableInventoryId())
                            .ne(ReferenceMaterial::getStatus, -4);
                }
                List<ReferenceMaterial> list1 = referenceMaterialService.list(referenceMaterialLambdaQueryWrapper);

                referenceMaterialList.addAll(list1);
            }
        }

        if (referenceMaterialList.size() != 0) {
            //提供多余字段
            for (ReferenceMaterial referenceMaterial : referenceMaterialList) {

                ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

                ReagentConsumableInventory byId = reagentConsumableInventoryService.getById(referenceMaterial.getReagentConsumableInventoryId());

                BeanUtils.copyProperties(byId, reagentConsumableInventoryFullVO);

                reagentConsumableInventoryFullVO.setNumber(referenceMaterial.getNumber());

                reagentConsumableInventoryFullVO.setBatchDetailsId(referenceMaterial.getBatchDetailsId());

                BatchDetails batchDetails = batchDetailsService.getById(referenceMaterial.getBatchDetailsId());

                SupplierInformation supplierInformation = supplierInformationService.getById(batchDetails.getSupplierId());

                LambdaQueryWrapper<StandardReserveSolution> standardReserveSolutionLambdaQueryWrapper = new LambdaQueryWrapper<>();

                standardReserveSolutionLambdaQueryWrapper.eq(StandardReserveSolution::getSolutionNumbering, referenceMaterial.getNumber());

                StandardReserveSolution one1 = standardReserveSolutionService.getOne(standardReserveSolutionLambdaQueryWrapper);

                reagentConsumableInventoryFullVO.setSupplierName(supplierInformation.getSupplierName());
                reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());
                reagentConsumableInventoryFullVO.setBatchNumber(batchDetails.getBatchNumber());
                reagentConsumableInventoryFullVO.setManufacturerId(batchDetails.getSupplierId());
                reagentConsumableInventoryFullVO.setPurchaseTime(batchDetails.getPurchaseTime());
                reagentConsumableInventoryFullVO.setReferenceMaterialId(referenceMaterial.getId());
                reagentConsumableInventoryFullVO.setReferenceMaterialStatus(referenceMaterial.getStatus());
                if (one1 != null) {

                    reagentConsumableInventoryFullVO.setConfigurationConcentration(one1.getConfigurationConcentration());
                    reagentConsumableInventoryFullVO.setConfigurationDate(one1.getConfigurationDate());
                    reagentConsumableInventoryFullVO.setSolutionNumbering(referenceMaterial.getNumber());
                    reagentConsumableInventoryFullVO.setValidityPeriod(one1.getValidityPeriod());
                }
                reagentConsumableInventoryFullVOList.add(reagentConsumableInventoryFullVO);
            }
        }

        PageUtils pageUtils = new PageUtils();

        Page pages = pageUtils.getPages(current, size, reagentConsumableInventoryFullVOList);

        return pages;
    }


    //分页查询试剂耗材
    @Override
    public Page<ReagentConsumableInventoryFullVO> getAllRM(Integer current, Integer size, String reagentConsumableName) {

        List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOList = new ArrayList<>();

        List<ReagentConsumableInventory> list = reagentConsumableInventoryService.list(Wrappers.<ReagentConsumableInventory>query()
                .eq("category", "试剂").or()
                .eq("category", "耗材")
                .like(StrUtil.isNotBlank(reagentConsumableName), "reagent_consumable_name", reagentConsumableName));
        //找出对应的仓库信息
            for (ReagentConsumableInventory reagentConsumableInventory : list) {

                LambdaQueryWrapper<BatchDetails> batchDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

                batchDetailsLambdaQueryWrapper
                        .eq(BatchDetails::getReagentConsumableInventoryId, reagentConsumableInventory.getReagentConsumableInventoryId())
                        .eq(BatchDetails::getServiceStatus, 1);

                List<BatchDetails> list1 = batchDetailsService.list(batchDetailsLambdaQueryWrapper);
                //将不同批次的物品展示出来
                for (BatchDetails batchDetails : list1) {

                    ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

                    BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryFullVO);

                    SupplierInformation byId = supplierInformationService.getById(batchDetails.getSupplierId());

                    reagentConsumableInventoryFullVO.setSupplierName(byId.getSupplierName());

                    reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());

                    reagentConsumableInventoryFullVO.setBatchDetailsId(batchDetails.getBatchDetailsId());

                    reagentConsumableInventoryFullVO.setTotalQuantity(batchDetails.getQuantity());

                    reagentConsumableInventoryFullVOList.add(reagentConsumableInventoryFullVO);
                }
            }
            PageUtils pageUtils = new PageUtils();

            Page pages = pageUtils.getPages(current, size, reagentConsumableInventoryFullVOList);

            return pages;
        }


    @Override
    public List<ReagentConsumableInventoryFullVO> getReagentConsumableInventoryFull(QueryWrapper<ReagentConsumableInventory> qw){
        List<ReagentConsumableInventory> list = reagentConsumableInventoryService.list(qw);
//        return list;

        List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOList = new ArrayList<>();

        //找出对应的仓库信息
        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            LambdaQueryWrapper<BatchDetails> batchDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

            batchDetailsLambdaQueryWrapper.eq(BatchDetails::getReagentConsumableInventoryId, reagentConsumableInventory.getReagentConsumableInventoryId())
                    .or().eq(BatchDetails::getServiceStatus, 1);

            List<BatchDetails> list1 = batchDetailsService.list(batchDetailsLambdaQueryWrapper);
            //将不同批次的物品展示出来
            for (BatchDetails batchDetails : list1) {

                ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

                BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryFullVO);

                SupplierInformation byId = supplierInformationService.getById(batchDetails.getSupplierId());

                reagentConsumableInventoryFullVO.setSupplierName(byId.getSupplierName());

                reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());

                reagentConsumableInventoryFullVO.setBatchDetailsId(batchDetails.getBatchDetailsId());

                reagentConsumableInventoryFullVO.setTotalQuantity(batchDetails.getQuantity());

                reagentConsumableInventoryFullVOList.add(reagentConsumableInventoryFullVO);
            }
        }
        return reagentConsumableInventoryFullVOList;
    }
}
