package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.StandardMaterialApprovalFormDTO;
import digital.laboratory.platform.reagent.entity.ReferenceMaterial;
import digital.laboratory.platform.reagent.entity.StandardMaterialApprovalForm;
import digital.laboratory.platform.reagent.entity.SupplierInformation;
import digital.laboratory.platform.reagent.mapper.StandardMaterialApprovalFormMapper;
import digital.laboratory.platform.reagent.service.ReferenceMaterialService;
import digital.laboratory.platform.reagent.service.StandardMaterialApprovalFormService;
import digital.laboratory.platform.reagent.service.SupplierInformationService;
import digital.laboratory.platform.reagent.vo.StandardMaterialApprovalFormVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * (标准物质停用/报废销毁/恢复/降级使用审批表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质停用 / 报废销毁 / 恢复 / 降级使用审批表) 服务实现类
 */
@Service
public class StandardMaterialApprovalFormServiceImpl extends ServiceImpl<StandardMaterialApprovalFormMapper, StandardMaterialApprovalForm> implements StandardMaterialApprovalFormService {

    @Autowired
    private StandardMaterialApprovalFormService standardMaterialApprovalFormService;

    @Autowired
    private SupplierInformationService supplierInformationService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Override
    public StandardMaterialApprovalForm commitById(StandardMaterialApprovalFormDTO standardMaterialApprovalFormDTO, DLPUser dlpUser) {

        StandardMaterialApprovalForm standardMaterialApprovalForm = new StandardMaterialApprovalForm();

        BeanUtils.copyProperties(standardMaterialApprovalFormDTO, standardMaterialApprovalForm);

        standardMaterialApprovalForm.setApplicantId(dlpUser.getId());

        standardMaterialApprovalForm.setApplicantName(dlpUser.getName());

        standardMaterialApprovalForm.setCommitStatus(1);

        standardMaterialApprovalForm.setId(IdWorker.get32UUID().toUpperCase());

        standardMaterialApprovalForm.setApprovalOfTime(LocalDateTime.now());

        standardMaterialApprovalForm.setCommitTime(LocalDateTime.now());

        if (standardMaterialApprovalFormDTO.getApplicationContent() == 1 | standardMaterialApprovalFormDTO.getApplicationContent() == 2) {

            ReferenceMaterial byId = referenceMaterialService.getById(standardMaterialApprovalFormDTO.getReferenceMaterialId());

            byId.setStatus(-1);

            referenceMaterialService.updateById(byId);

        } else if (standardMaterialApprovalFormDTO.getApplicationContent() == 4) {

            ReferenceMaterial byId = referenceMaterialService.getById(standardMaterialApprovalFormDTO.getReferenceMaterialId());

            byId.setStatus(-2);

            referenceMaterialService.updateById(byId);


        } else {

            ReferenceMaterial byId = referenceMaterialService.getById(standardMaterialApprovalFormDTO.getReferenceMaterialId());

            byId.setStatus(0);

            referenceMaterialService.updateById(byId);

        }


        if (standardMaterialApprovalFormService.save(standardMaterialApprovalForm)) {
            return standardMaterialApprovalForm;
        } else throw new RuntimeException(String.format("保存失败"));
    }


    @Override
    public IPage<StandardMaterialApprovalFormVO> getVOPage(IPage<StandardMaterialApprovalForm> page, QueryWrapper<StandardMaterialApprovalForm> qw) {

        IPage<StandardMaterialApprovalFormVO> voPage = baseMapper.getVOPage(page, qw);

        return voPage;
    }

    @Override
    public StandardMaterialApprovalFormVO getVO(String standardMaterialApprovalFormId) {

        StandardMaterialApprovalFormVO vo = baseMapper.getVO(standardMaterialApprovalFormId);

        SupplierInformation byId1 = supplierInformationService.getById(vo.getManufacturerId());

        vo.setManufacturerName(byId1.getSupplierName());

        return vo;
    }

    @Override//一级审核
    public StandardMaterialApprovalForm auditPrimary(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        StandardMaterialApprovalForm byId = standardMaterialApprovalFormService.getById(auditAndApproveDTO.getUuId());

        byId.setAuditOpinionOfPrimary(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditResultOfPrimary(auditAndApproveDTO.getAuditResult());
        byId.setPrimaryAuditorId(dlpUser.getId());
        byId.setAuditTimeOfPrimary(LocalDateTime.now());

        if (auditAndApproveDTO.getAuditResult() == true) {
            byId.setCommitStatus(2);
        } else {
            byId.setCommitStatus(-2);
        }

        if (standardMaterialApprovalFormService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("审核失败"));
    }

    @Override//二级审核
    public StandardMaterialApprovalForm auditSecondary(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        StandardMaterialApprovalForm byId = standardMaterialApprovalFormService.getById(auditAndApproveDTO.getUuId());

        byId.setAuditOpinionOfSecondary(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditResultOfSecondary(auditAndApproveDTO.getAuditResult());
        byId.setSecondaryAuditorId(dlpUser.getId());
        byId.setAuditTimeOfSecondary(LocalDateTime.now());
        if (auditAndApproveDTO.getAuditResult() == true) {
            byId.setCommitStatus(3);
        } else {
            byId.setCommitStatus(-3);
        }
        if (standardMaterialApprovalFormService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("审核失败"));
    }

    @Override//审批
    public StandardMaterialApprovalForm approveById(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        StandardMaterialApprovalForm byId = standardMaterialApprovalFormService.getById(auditAndApproveDTO.getUuId());

        byId.setOpinionOfApproval(auditAndApproveDTO.getAuditOpinion());
        byId.setResultOfApproval(auditAndApproveDTO.getAuditResult());
        byId.setApproverId(dlpUser.getId());
        byId.setApprovalOfTime(LocalDateTime.now());
        if (auditAndApproveDTO.getApproveResult() == true) {
            byId.setCommitStatus(4);
        } else {
            byId.setCommitStatus(-4);
        }
        if (standardMaterialApprovalFormService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("审批失败"));
    }
}
