package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.StandardReserveSolutionDTO;
import digital.laboratory.platform.reagent.dto.StandardReserveSolutionFullDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.StandardReserveSolutionMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.LocationVO;
import digital.laboratory.platform.reagent.vo.SolutionUseFormVO;
import digital.laboratory.platform.reagent.vo.StandardReserveSolutionFullVO;
import digital.laboratory.platform.reagent.vo.StandardReserveSolutionVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * (标准储备溶液配制及使用记录表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准储备溶液配制及使用记录表) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class StandardReserveSolutionServiceImpl extends ServiceImpl<StandardReserveSolutionMapper, StandardReserveSolution> implements StandardReserveSolutionService {

    @Autowired
    private StandardReserveSolutionService standardReserveSolutionService;

    @Autowired
    private SolutionUseFormService solutionUseFormService;

    @Autowired
    private LatticeFormService latticeFormService;

    @Autowired
    private StorageRoomFormService storageRoomFormService;

    @Autowired
    private CabinetFormService cabinetFormService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;
    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private BatchDetailsService batchDetailsService;

    @Override//通过ID查询标准溶液的配置记录
    public StandardReserveSolutionVO getStandardReserveSolutionVOById(String id) {

        StandardReserveSolutionVO standardReserveSolutionVOById = baseMapper.getStandardReserveSolutionVOById(id);
        //查出该标准储备溶液对应的所有使用记录
        List<SolutionUseFormVO> solutionUseFormVOList = solutionUseFormService.getSolutionUseFormVOList(standardReserveSolutionVOById.getId());

        standardReserveSolutionVOById.setSolutionUseFormVOList(solutionUseFormVOList);

        return standardReserveSolutionVOById;
    }

    @Override//配置标准储备溶液
    @Transactional
    public StandardReserveSolutionVO addById(StandardReserveSolutionDTO standardReserveSolutionDTO, DLPUser dlpUser) {

        StandardReserveSolution standardReserveSolution = new StandardReserveSolution();

        BeanUtils.copyProperties(standardReserveSolutionDTO, standardReserveSolution);

        standardReserveSolution.setId(IdWorker.get32UUID().toUpperCase());

        standardReserveSolution.setDispenserId(dlpUser.getId());

        standardReserveSolution.setConfigurationDate(LocalDateTime.now());

        standardReserveSolution.setStatus(0);

        LambdaQueryWrapper<ReagentConsumables> reagentConsumablesLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumablesLambdaQueryWrapper.eq(ReagentConsumables::getReagentConsumableName, standardReserveSolution.getSolutionName());
        reagentConsumablesLambdaQueryWrapper.eq(ReagentConsumables::getConfigurationConcentration, standardReserveSolution.getConfigurationConcentration());

        ReagentConsumables one = reagentConsumablesService.getOne(reagentConsumablesLambdaQueryWrapper);
        //判断是否存在过该标准储备溶液，若未存在，则将其存入试剂耗材类
        if (one == null) {

            ReagentConsumables reagentConsumables = new ReagentConsumables();

            ReferenceMaterial byId = referenceMaterialService.getById(standardReserveSolution.getReferenceMaterialId());

            ReagentConsumables reagentConsumables1 = reagentConsumablesService.getById(byId.getReagentConsumableId());

            BeanUtils.copyProperties(reagentConsumables1, reagentConsumables);

            reagentConsumables1.setReagentConsumableId(IdWorker.get32UUID().toUpperCase());
            reagentConsumables1.setReagentConsumableName(standardReserveSolution.getSolutionName());
            reagentConsumables1.setConfigurationConcentration(standardReserveSolution.getConfigurationConcentration());
            reagentConsumables1.setCategory("标准储备溶液");
            reagentConsumables1.setCreateTime(LocalDateTime.now());
            reagentConsumables1.setUpdateTime(LocalDateTime.now());

            reagentConsumablesService.save(reagentConsumables1);
        }

        if (standardReserveSolutionService.save(standardReserveSolution)) {

            StandardReserveSolutionVO standardReserveSolutionVOById = standardReserveSolutionService.getStandardReserveSolutionVOById(standardReserveSolution.getId());

            return standardReserveSolutionVOById;
        } else throw new RuntimeException(String.format("配置失败"));
    }

    @Override//标准储备溶液入库任务
    public IPage<StandardReserveSolutionVO> getStandardReserveSolutionVOList(IPage<StandardReserveSolution> page, QueryWrapper<StandardReserveSolution> qw) {

        IPage<StandardReserveSolutionVO> standardReserveSolutionVOPage = baseMapper.getStandardReserveSolutionVOPage(page, qw);

        return standardReserveSolutionVOPage;
    }

    @Override//标准储备溶液配置入库
    @Transactional
    public StandardReserveSolutionFullVO warehousingById(StandardReserveSolutionFullDTO standardReserveSolutionFullDTO) {

        String id = standardReserveSolutionFullDTO.getId();

        String latticeId = standardReserveSolutionFullDTO.getLatticeId();

        String warehousingRemarks = standardReserveSolutionFullDTO.getWarehousingRemarks();
        //提供格子位置与标准储备溶液配置表ID
        StandardReserveSolution byId = standardReserveSolutionService.getById(id);

        ReferenceMaterial byId1 = referenceMaterialService.getById(byId.getReferenceMaterialId());

        BatchDetails batchDetails = batchDetailsService.getById(byId1.getBatchDetailsId());

        LambdaQueryWrapper<ReagentConsumables> reagentConsumablesLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumablesLambdaQueryWrapper.eq(ReagentConsumables::getReagentConsumableName, byId.getSolutionName());

        reagentConsumablesLambdaQueryWrapper.eq(ReagentConsumables::getConfigurationConcentration, byId.getConfigurationConcentration());
        //查询出该标准储备溶液的对象
        ReagentConsumables reagentConsumables = reagentConsumablesService.getOne(reagentConsumablesLambdaQueryWrapper);

        LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, reagentConsumables.getReagentConsumableId());
        //判断仓库是否存在过该标准储备溶液 （标准储备溶液ID与标准物质共用一个RMID，但标准储备溶液具有）
        ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);
        //若不存在，则创建一条该标准储备溶液的仓库信息记录表
        if (one == null) {

            ReagentConsumableInventory reagentConsumableInventory = new ReagentConsumableInventory();

            BeanUtils.copyProperties(reagentConsumables, reagentConsumableInventory);

            reagentConsumableInventory.setReagentConsumableInventoryId(IdWorker.get32UUID().toUpperCase());

            reagentConsumableInventory.setStatus(1);

            reagentConsumableInventory.setConfigurationConcentration(byId.getConfigurationConcentration());
            reagentConsumableInventory.setCreateTime(LocalDateTime.now());
            reagentConsumableInventory.setUpdateTime(LocalDateTime.now());

            ReferenceMaterial referenceMaterial = new ReferenceMaterial();
            //获取当前年月日
            Date date = new Date();

            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");

            formatter.format(date);
            //创建标准溶液对象，并生成编号
            referenceMaterial.setStatus(0);
            referenceMaterial.setId(IdWorker.get32UUID().toUpperCase());
            referenceMaterial.setReagentConsumableId(reagentConsumableInventory.getReagentConsumableId());
            referenceMaterial.setReagentConsumableInventoryId(reagentConsumableInventory.getReagentConsumableInventoryId());
            referenceMaterial.setNumber(reagentConsumableInventory.getEnglishName() + "-" + formatter.format(date));
            referenceMaterial.setBatchDetailsId(batchDetails.getBatchDetailsId());
            referenceMaterial.setLatticeId(latticeId);
            referenceMaterial.setLocation(standardReserveSolutionFullDTO.getLocation());

            referenceMaterialService.save(referenceMaterial);

            reagentConsumableInventory.setTotalQuantity(1);
            reagentConsumableInventory.setStorageLife(byId.getValidityPeriod());

            batchDetails.setReagentConsumableInventoryId(reagentConsumableInventory.getReagentConsumableInventoryId());
            batchDetails.setLatticeId(latticeId);

            byId.setSolutionNumbering(referenceMaterial.getNumber());

            byId.setStatus(1);

            byId.setWarehousingRemarks(warehousingRemarks);

            if (reagentConsumableInventoryService.save(reagentConsumableInventory) && standardReserveSolutionService
                    .updateById(byId)) {
                StandardReserveSolutionFullVO byFullVOId = standardReserveSolutionService.getByFullVOId(byId.getId());
                return byFullVOId;
            } else {
                throw new RuntimeException(String.format("入库失败"));
            }

        } else {

            ReferenceMaterial referenceMaterial = new ReferenceMaterial();
            //获取当前年月日
            Date date = new Date();

            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-mm-dd");

            formatter.format(date);
            //创建标准溶液对象，并生成编号
            referenceMaterial.setStatus(0);
            referenceMaterial.setId(IdWorker.get32UUID().toUpperCase());
            referenceMaterial.setReagentConsumableId(one.getReagentConsumableId());
            referenceMaterial.setReagentConsumableInventoryId(one.getReagentConsumableInventoryId());
            referenceMaterial.setNumber(one.getEnglishName() + "-" + formatter.format(date));
            referenceMaterial.setBatchDetailsId(batchDetails.getBatchDetailsId());
            referenceMaterial.setLatticeId(latticeId);
            referenceMaterial.setLocation(standardReserveSolutionFullDTO.getLocation());

            referenceMaterialService.save(referenceMaterial);
            //将存储信息录入仓库表

            one.setTotalQuantity(one.getTotalQuantity() + 1);
            one.setStorageLife(byId.getValidityPeriod());
        }

        byId.setStatus(1);

        byId.setWarehousingRemarks(warehousingRemarks);


        if (standardReserveSolutionService.updateById(byId) && reagentConsumableInventoryService.updateById(one)) {

            StandardReserveSolutionFullVO byFullVOId = standardReserveSolutionService.getByFullVOId(byId.getId());
            return byFullVOId;
        } else {
            throw new RuntimeException(String.format("入库失败"));
        }
    }

    @Override//通过ID查找已入库的标准储备溶液信息
    public StandardReserveSolutionFullVO getByFullVOId(String id) {

        StandardReserveSolutionVO standardReserveSolutionVOById = standardReserveSolutionService.getStandardReserveSolutionVOById(id);

        StandardReserveSolutionFullVO standardReserveSolutionFullVO = new StandardReserveSolutionFullVO();

        LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

        ReferenceMaterial byId = referenceMaterialService.getById(standardReserveSolutionVOById.getReferenceMaterialId());

        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableName, standardReserveSolutionVOById.getSolutionName());
        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getConfigurationConcentration, standardReserveSolutionVOById.getConfigurationConcentration());

        ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);

        return standardReserveSolutionFullVO;
    }

}
