package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.StandardSolutionCurveDTO;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.entity.ReferenceMaterial;
import digital.laboratory.platform.reagent.entity.StandardSolutionCurve;
import digital.laboratory.platform.reagent.mapper.StandardSolutionCurveMapper;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import digital.laboratory.platform.reagent.service.ReferenceMaterialService;
import digital.laboratory.platform.reagent.service.StandardSolutionCurveService;
import digital.laboratory.platform.reagent.vo.StandardSolutionCurveVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * (标准工作曲线溶液配置记录表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准工作曲线溶液配置记录表) 服务实现类
 */
@Service
public class StandardSolutionCurveServiceImpl extends ServiceImpl<StandardSolutionCurveMapper, StandardSolutionCurve> implements StandardSolutionCurveService {

    @Autowired
    private StandardSolutionCurveService standardSolutionCurveService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Override
    public IPage<StandardSolutionCurveVO> getStandardSolutionCurveVOPage(IPage<StandardSolutionCurve> page, QueryWrapper<StandardSolutionCurve> qw){

        IPage<StandardSolutionCurveVO> standardSolutionCurveVOPage = baseMapper.getStandardSolutionCurveVOPage(page, qw);

        return standardSolutionCurveVOPage;
    }

    @Override
    public StandardSolutionCurve addById(StandardSolutionCurveDTO standardSolutionCurveDTO, DLPUser dlpUser){

        StandardSolutionCurve standardSolutionCurve = new StandardSolutionCurve();

        String solutionNumbering = standardSolutionCurveDTO.getSolutionNumbering();

        LambdaQueryWrapper<ReferenceMaterial> referenceMaterialLambdaQueryWrapper = new LambdaQueryWrapper<>();

        referenceMaterialLambdaQueryWrapper.eq(ReferenceMaterial::getNumber,solutionNumbering);

        ReferenceMaterial referenceMaterial = referenceMaterialService.getOne(referenceMaterialLambdaQueryWrapper);

        standardSolutionCurve.setId(IdWorker.get32UUID().toUpperCase());

        ReagentConsumables reagentConsumables = reagentConsumablesService.getById(referenceMaterial.getReagentConsumableId());

        standardSolutionCurve.setSolutionName(reagentConsumables.getReagentConsumableName());
        standardSolutionCurve.setSolutionConcentration(reagentConsumables.getConfigurationConcentration());
        standardSolutionCurve.setConfigurationDate(LocalDateTime.now());
        standardSolutionCurve.setConcentrationNumbering(referenceMaterial.getNumber());
        standardSolutionCurve.setStepSolutionConcentration(standardSolutionCurveDTO.getStepSolutionConcentration());
        standardSolutionCurve.setReferenceMaterialId(referenceMaterial.getId());
        standardSolutionCurve.setDispenserId(dlpUser.getId());

        if (standardSolutionCurveService.save(standardSolutionCurve)){
            return standardSolutionCurve;
        }else throw new RuntimeException(String.format("配置失败"));

    }
}
